package com.yame;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.mxgraph.io.mxCodec;
import com.mxgraph.model.mxCell;
import com.mxgraph.model.mxGraphModel;
import com.mxgraph.model.mxICell;
import com.mxgraph.model.mxIGraphModel;
import com.mxgraph.util.mxUtils;
import com.mxgraph.util.mxXmlUtils;
import com.mxgraph.view.mxGraph;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Graph
 */
public class Graph {

    private final Logger log = LoggerFactory.getLogger(Graph.class.getName());

    mxGraph graph;
    mxIGraphModel imodel;

    private static class mxCellCloned {

        mxCell cell;

        String parentid;
        String sourceid;
        String targetid;
        String newId;
        boolean isGroup;

        public static List<mxCellCloned> clone(final Graph graph, Boolean isInGroup) throws CloneNotSupportedException {
            final String uid = Utils.GeneratorUID();
            final HashMap<String, mxCellCloned> idmap = new HashMap<>();

            final Object[] _cells = graph.getMXGraph().getChildCells(graph.getMXGraph().getDefaultParent());
            final List<mxCellCloned> cells = new ArrayList<mxCellCloned>();

            mxCellCloned groupCell = null;

            if (isInGroup) {
                groupCell = new mxCellCloned();
                groupCell.cell = new mxCell();
                groupCell.cell.setId(uid);
                groupCell.cell.setStyle("group");
                groupCell.cell.setConnectable(false);
                groupCell.cell.setVertex(true);
                groupCell.isGroup = true;
            }

            for (final Object _cell : _cells) {
                final mxCell cell = (mxCell) _cell;
                final String curid = cell.getId();

                if (curid.equals("0") || curid.equals("1")) {
                    continue;
                }

                final mxCellCloned cloned = new mxCellCloned();
                cloned.cell = (mxCell) cell.clone();

                cloned.cell.setId(curid);
                final mxICell parent = cell.getParent();
                if (parent != null) {
                    final String pid = parent.getId();
                    if(!pid.equals("1")){
                        cloned.parentid = pid;
                    }
                    
                }

                final mxICell source = cell.getSource();
                if (source != null) {
                    final String sid = source.getId();
                    cloned.sourceid = sid;
                }

                final mxICell target = cell.getTarget();
                if (target != null) {
                    final String tid = target.getId();
                    cloned.targetid = tid;
                }
                cells.add(cloned);
            }

            for (final mxCellCloned cloned : cells) {
                final String curid = cloned.cell.getId();
                final String[] ids = curid.split("-");
                cloned.newId = uid + "-" + ids[ids.length - 1];
                idmap.put(curid, cloned);
                cloned.cell.setId(cloned.newId);
            }

            for (final mxCellCloned cloned : cells) {

                if (cloned.sourceid != null) {
                    final mxCellCloned source = idmap.get(cloned.sourceid);
                    cloned.cell.setSource(source.cell);
                }

                if (cloned.targetid != null) {
                    final mxCellCloned target = idmap.get(cloned.targetid);
                    cloned.cell.setTarget(target.cell);
                }

                if (cloned.parentid != null) {
                    final mxCellCloned parent = idmap.get(cloned.parentid);
                    if (parent != null) {
                        cloned.cell.setParent(parent.cell);
                    }
                } else {
                    if(isInGroup) {
                        cloned.cell.setParent(groupCell.cell);
                        mxICell parent = cloned.cell.getParent();
                        System.out.println(parent);
                    }
                }

            }

            if (isInGroup) {
                cells.add(groupCell);
            }

            return cells;
        }

        public static List<mxCellCloned> clone(final Graph graph) throws CloneNotSupportedException {
            return clone(graph, false);
        }
    }

    public Graph() {
        imodel = new mxGraphModel();
        graph = new mxGraph(imodel);
    }

    public static Graph merge(final Graph g1, final Graph g2, boolean isInGroup) throws CloneNotSupportedException {
        final Graph graph = new Graph();
        graph.imodel.beginUpdate();

        final List<mxCellCloned> cellCloneds1 = mxCellCloned.clone(g1, isInGroup);
        final List<mxCellCloned> cellCloneds2 = mxCellCloned.clone(g2, isInGroup);


        HashMap<String, Object> groupmap = new HashMap<String, Object>();

        cellCloneds1.forEach(cloned -> {

            mxICell parent = cloned.cell.getParent();
            if (parent != null) {
                
                Object np = groupmap.get(parent.getId());
                if(np != null) {
                    graph.getMXGraph().addCell(cloned.cell, np);
                } else {
                    System.out.println("x");
                }
                
            } else {
                Object groupcell = graph.getMXGraph().addCell(cloned.cell);
                groupmap.put(cloned.cell.getId(), groupcell);
            }

        });

        cellCloneds2.forEach(cloned -> {
            if (cloned.cell.getParent() != null) {
                graph.getMXGraph().addCell(cloned.cell, cloned.cell.getParent());
            } else {
                graph.getMXGraph().addCell(cloned.cell);
            }
        });

        graph.imodel.endUpdate();
        return graph;
    }

    public static Graph merge(final Graph g1, final Graph g2) throws CloneNotSupportedException {
        return merge(g1, g2, false);
    }

    private void loadString(final String xmlString) {

        imodel = new mxGraphModel();
        graph = new mxGraph(imodel);

        final Document doc = mxXmlUtils.parseXml(xmlString);
        final mxCodec codec = new mxCodec(doc);
        final Node dmodel = doc.getElementsByTagName("mxGraphModel").item(0);
        codec.decode(dmodel, imodel);

    }

    public void load(final String pathStr) throws IOException {

        final Path path = Paths.get(pathStr);
        final String xmlString = Files.readString(path);
        loadString(xmlString);

    }

    public void load(final byte[] xmlBytes) {

        loadString(new String(xmlBytes));

    }

    public mxGraph getMXGraph() {
        return this.graph;
    }

    public String dumpModel() {
        return mxXmlUtils.getXml(new mxCodec().encode(imodel));
    }
}